@@ -1,8 +1,8 @@
 """AwesomeVersion."""
 from __future__ import annotations
 
-from types import TracebackType
-from typing import Any, Type
+from typing import TYPE_CHECKING, Any, Dict
+from warnings import warn
 
 from .comparehandlers.container import compare_handler_container
 from .comparehandlers.modifier import compare_handler_semver_modifier
@@ -15,7 +15,6 @@ from .strategy import (
     AwesomeVersionStrategy,
     AwesomeVersionStrategyDescription,
 )
-from .typing import EnsureStrategyIterableType, EnsureStrategyType, VersionType
 from .utils.regex import (
     RE_DIGIT,
     RE_MODIFIER,
@@ -24,6 +23,9 @@ from .utils.regex import (
     generate_full_string_regex,
 )
 
+if TYPE_CHECKING:
+    from .typing import EnsureStrategyIterableType, EnsureStrategyType, VersionType
+
 
 class _AwesomeVersionBase(str):
     """Base class for AwesomeVersion to allow the usage of the default JSON encoder."""
@@ -46,9 +48,13 @@ class AwesomeVersion(_AwesomeVersionBase):
     """
     AwesomeVersion class.
 
+    **args**:
+
     version:
         The version to create a AwesomeVersion object from
 
+    **kwargs**:
+
     ensure_strategy:
         Match the AwesomeVersion object against spesific
         strategies when creating if. If it does not match
@@ -69,19 +75,44 @@ class AwesomeVersion(_AwesomeVersionBase):
     _ensure_strategy: EnsureStrategyIterableType = []
 
     def __init__(
-        self,
+        self,  # pylint: disable=unused-argument
         version: VersionType,
+        *args: Any,
         ensure_strategy: EnsureStrategyType = None,
         find_first_match: bool = False,
+        **kwargs: Any,
     ) -> None:
         """Initialize AwesomeVersion."""
-        if isinstance(version, AwesomeVersion):
-            self._version = version._version
-        else:
-            self._version = str(version)
+        if args:
+            warn(
+                "Positional argument for ensure_strategy is deprecated. "
+                "This will be removed in 2023, use keyword argument instead",
+                category=DeprecationWarning,
+                stacklevel=2,
+            )
+            ensure_strategy = args[0]
+            if len(args) == 2:
+                warn(
+                    "Positional argument for find_first_match is deprecated. "
+                    "This will be removed in 2023, use keyword argument instead",
+                    category=DeprecationWarning,
+                    stacklevel=2,
+                )
+                find_first_match = args[1]
+
+        self._version = (
+            version._version if isinstance(version, AwesomeVersion) else str(version)
+        )
+
         if isinstance(self._version, str):
             self._version = self._version.strip()
 
+        if find_first_match and not ensure_strategy:
+            warn(
+                "Can not use find_first_match without ensure_strategy",
+                stacklevel=2,
+            )
+
         if ensure_strategy is not None:
             self._ensure_strategy = ensure_strategy = (
                 ensure_strategy
@@ -115,12 +146,7 @@ class AwesomeVersion(_AwesomeVersionBase):
     def __enter__(self) -> AwesomeVersion:
         return self
 
-    def __exit__(
-        self,
-        exctype: Type[BaseException] | None,
-        excinst: BaseException | None,
-        exctb: TracebackType | None,
-    ) -> bool:
+    def __exit__(self, *_: Any, **__: Any) -> None:
         pass
 
     def __repr__(self) -> str:
@@ -178,6 +204,37 @@ class AwesomeVersion(_AwesomeVersionBase):
     def __ge__(self, compareto: object) -> bool:
         return self.__eq__(compareto) or self.__gt__(compareto)
 
+    def __sub__(self, compareto: object) -> AwesomeVersionDiff:
+        return self.diff(compareto)
+
+    def diff(self, compareto: VersionType) -> AwesomeVersionDiff:
+        """Return a dictionary with differences between 2 AwesomeVersion objects."""
+        if isinstance(compareto, (str, float, int)):
+            compareto = AwesomeVersion(compareto)
+        if not isinstance(compareto, AwesomeVersion):
+            raise AwesomeVersionCompareException("Not a valid AwesomeVersion object")
+        return AwesomeVersionDiff(
+            {
+                "major": self.major != compareto.major,
+                "minor": self.minor != compareto.minor,
+                "patch": self.patch != compareto.patch,
+                "modifier": self.modifier != compareto.modifier,
+                "strategy": self.strategy != compareto.strategy,
+            }
+        )
+
+    def in_range(self, lowest: VersionType, highest: VersionType) -> bool:
+        """Check if version is in range."""
+        if isinstance(lowest, (str, float, int)):
+            lowest = AwesomeVersion(lowest)
+        if isinstance(highest, (str, float, int)):
+            highest = AwesomeVersion(highest)
+        if not isinstance(lowest, AwesomeVersion):
+            raise AwesomeVersionCompareException("Lowest version is not valid")
+        if not isinstance(highest, AwesomeVersion):
+            raise AwesomeVersionCompareException("Highest version is not valid")
+        return lowest <= self <= highest
+
     def section(self, idx: int) -> int:
         """Return the value of the specified section of the version."""
         if self.sections >= (idx + 1):
@@ -187,7 +244,7 @@ class AwesomeVersion(_AwesomeVersionBase):
         return 0
 
     @staticmethod
-    def _compare_versions(version_a: str, version_b: str) -> bool:
+    def _compare_versions(version_a: AwesomeVersion, version_b: AwesomeVersion) -> bool:
         """Compare versions."""
         for handler in (
             compare_handler_container,
@@ -195,14 +252,14 @@ class AwesomeVersion(_AwesomeVersionBase):
             compare_handler_sections,
             compare_handler_semver_modifier,
         ):
-            result = handler(AwesomeVersion(version_a), AwesomeVersion(version_b))
+            result = handler(version_a, version_b)
             if result is not None:
                 return result
         return False
 
     @property
     def string(self) -> str:
-        """Return a string representaion of the version."""
+        """Return a string representation of the version."""
         if not self._version:
             return self._version
 
@@ -245,7 +302,7 @@ class AwesomeVersion(_AwesomeVersionBase):
 
     @property
     def sections(self) -> int:
-        """Return a int representaion of the number of sections in the version."""
+        """Return a int representation of the number of sections in the version."""
         if self._sections is not None:
             return self._sections
 
@@ -267,11 +324,12 @@ class AwesomeVersion(_AwesomeVersionBase):
         """
         Return a AwesomeVersion representation of the major version.
 
-        Will return None if the versions is not semver/buildver/simplever.
+        Will return None if the versions is not semver/buildver/calver/simplever.
         """
         if self.strategy not in (
             AwesomeVersionStrategy.SEMVER,
             AwesomeVersionStrategy.BUILDVER,
+            AwesomeVersionStrategy.CALVER,
             AwesomeVersionStrategy.SIMPLEVER,
         ):
             return None
@@ -282,11 +340,12 @@ class AwesomeVersion(_AwesomeVersionBase):
         """
         Return a AwesomeVersion representation of the minor version.
 
-        Will return None if the versions is not semver/simplever
+        Will return None if the versions is not semver/simplever/calver
         Will return None if the version does not have at least 2 sections.
         """
         if self.strategy not in (
             AwesomeVersionStrategy.SEMVER,
+            AwesomeVersionStrategy.CALVER,
             AwesomeVersionStrategy.SIMPLEVER,
         ):
             return None
@@ -298,13 +357,14 @@ class AwesomeVersion(_AwesomeVersionBase):
         """
         Return a AwesomeVersion representation of the patch version.
 
-        Will return None if the versions is not semver/simplever
+        Will return None if the versions is not semver/simplever/calver
         Will return None if the version does not have at least 3 sections.
         """
         if (
             self.strategy
             not in (
                 AwesomeVersionStrategy.SEMVER,
+                AwesomeVersionStrategy.CALVER,
                 AwesomeVersionStrategy.SIMPLEVER,
             )
             or self.sections < 3
@@ -312,6 +372,16 @@ class AwesomeVersion(_AwesomeVersionBase):
             return None
         return AwesomeVersion(self.section(2))
 
+    @property
+    def micro(self) -> AwesomeVersion | None:
+        """Alias to self.patch"""
+        return self.patch
+
+    @property
+    def year(self) -> AwesomeVersion | None:
+        """Alias to self.major, here to provide a better name for use in CalVer."""
+        return self.major
+
     @property
     def valid(self) -> bool:
         """Return True if the version is not UNKNOWN."""
@@ -393,3 +463,42 @@ class AwesomeVersion(_AwesomeVersionBase):
                 generate_full_string_regex(RE_SIMPLE).match(self.string) is not None
             )
         return self._simple
+
+
+class AwesomeVersionDiff:
+    """Structured output of AwesomeVersion.diff"""
+
+    def __init__(self, changes: Dict[str, bool]) -> None:
+        """Initialize the AwesomeVersionDiff."""
+        self._changes = changes
+
+    def __repr__(self) -> str:
+        return (
+            f"AwesomeVersionDiff(major={self.major}, minor={self.minor}, "
+            f"patch={self.patch}, modifier={self.modifier}, strategy={self.strategy})"
+        )
+
+    @property
+    def major(self) -> bool:
+        """Return True if the major version has changed."""
+        return self._changes["major"]
+
+    @property
+    def minor(self) -> bool:
+        """Return True if the minor version has changed."""
+        return self._changes["minor"]
+
+    @property
+    def patch(self) -> bool:
+        """Return True if the patch version has changed."""
+        return self._changes["patch"]
+
+    @property
+    def modifier(self) -> bool:
+        """Return True if the modifier version has changed."""
+        return self._changes["modifier"]
+
+    @property
+    def strategy(self) -> bool:
+        """Return True if the strategy has changed."""
+        return self._changes["strategy"]
